@@ -22,6 +22,10 @@ class OptionClass:
 
 class ImproviserMainWindow(QtGui.QMainWindow):
 
+	lastfile = ""
+	lastmidi = ""
+
+
 	def __init__(self):
 		QtGui.QMainWindow.__init__(self)
 
@@ -70,6 +74,17 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		self.connect(self.ui.action_About,
 			QtCore.SIGNAL("activated()"),
 			self.show_about)
+		self.connect(self.ui.action_Load,
+			QtCore.SIGNAL("activated()"),
+			self.load_dialog)
+		self.connect(self.ui.action_Save,
+			QtCore.SIGNAL("activated()"),
+			self.save_file)
+		self.connect(self.ui.actionS_ave_as,
+			QtCore.SIGNAL("activated()"),
+			self.save_dialog)
+		
+			
 
 	def set_defaults(self):
 		self.ui.resolution.setCurrentIndex(3) # '8'
@@ -139,31 +154,115 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		w.activateWindow()
 		w.exec_()
 
-	def start_simulation(self):
+	def load_dialog(self):
+		f = QtGui.QFileDialog()
+		s = f.getOpenFileName(self, "Load improviser file.",
+				"/home", "Improviser files (*.imp)")
+		if s != "":
+			o = Options.load_options_from_file(s, OptionClass())
+			self.set_options(o)
+
+			
+
+
+	def save_file(self):
+		if self.lastfile == "":
+			self.save_dialog()
+		else:
+			try:
+				Options.save_options(self.lastfile, 
+					self.get_options())
+			except IOError:
+				print "Couldn't open file for writing"
+
+	def save_dialog(self):
+		f = QtGui.QFileDialog()
+		s = str(f.getSaveFileName(self, "Save improviser file.",
+				"/home", "Improviser files (*.imp)"))
+		if s != "":
+			if s[-4:].lower() != ".imp":
+				filename = s + ".imp"
+			else:
+				filename = s
+			try:
+				Options.save_options(filename, 
+					self.get_options())
+				self.lastfile = filename
+			except IOError:
+				print "Couldn't open file for writing"
+
+	def set_options(self, options):
+		o = options
+
+		# LineEdit
+		self.ui.soundfont.setText(o.SF2)
+		self.ui.projectname.setText(o.project)
+
+		# SpinBoxes
+		self.ui.bpm.setValue(o.bpm)
+		self.ui.duration.setValue(o.duration)
+		self.ui.loop.setValue(o.loop)
+		self.ui.wildness.setValue(o.wild)
+
+		# ComboBoxes
+		i = self.combo_index_by_text(self.ui.resolution, str(o.resolution))
+		if i != -1:
+			self.ui.resolution.setCurrentIndex(i)
+		i = self.combo_index_by_text(self.ui.key, o.key)
+		if i != -1:
+			self.ui.key.setCurrentIndex(i)
+
+		# CheckBoxes
+		self.ui.swing.setChecked(bool(o.swing))
+		self.ui.no_fluidsynth.setChecked(bool(o.no_fluidsynth))
+
+
+
+	def get_options(self):
 		o = OptionClass()
-		o.SF2 = str(self.ui.soundfont.text())
+
+		# ComboBoxes
 		o.resolution = int(str(self.ui.resolution.currentText()))
 		o.key = str(self.ui.key.currentText())
+
+		# SpinBoxes
 		o.bpm = int(self.ui.bpm.value())
 		o.duration = int(self.ui.duration.value())
 		o.loop = int(self.ui.loop.value())
 		o.wild = float(self.ui.wildness.value())
-		o.swing = self.ui.swing.checkState()
-		o.no_fluidsynth = self.ui.no_fluidsynth.checkState()
+
+		# Checkboxes
+		o.swing = int(self.ui.swing.isChecked())
+		o.no_fluidsynth = int(self.ui.no_fluidsynth.isChecked())
+
 		o.instrument = self.get_instruments()
 		o.progression = self.get_progressions()
 		o.blocks = self.get_blocks()
+
+		# LineEdit
+		o.SF2 = str(self.ui.soundfont.text())
 		project = str(self.ui.projectname.text())
 		if project == "":
 			project = "Untitled"
-		o.midifile = self.get_file_name(project)
+		o.project = project
+		o.midifile = self.get_file_name(o.project)
+		return o
 
+	def combo_index_by_text(self, combo, text):
+		for x in range(combo.count()):
+			if combo.itemText(x) == text:
+				return x
+		return -1
+	
+	def start_simulation(self):
+		o = self.get_options()
 
 		try:
 			seq = Options.get_sequencer(o)
 			self.ui.stopbutton.setEnabled(True)
 			self.ui.startbutton.setEnabled(False)
 			seq.play()
+			self.lastmidi = o.midifile
 		except Options.OptionError, err:
 			print err
 
